@@ -14,14 +14,14 @@ String::Copyright - Representation of text-based copyright statements
 
 =head1 VERSION
 
-Version 0.001005
+Version 0.002000
 
 =cut
 
-our $VERSION = '0.001005';
+our $VERSION = '0.002000';
 
 # Dependencies
-use Exporter 5.57 (qw/import/);
+use parent 'Exporter::Tiny';
 use Carp ();
 
 our @EXPORT = qw/copyright/;
@@ -75,21 +75,46 @@ use overload (
 L<String::Copyright> Parses common styles of copyright statements
 and serializes in normalized format.
 
+=head1 OPTIONS
+
+Options can be set as an argument to the 'use' statement.
+
+    use String::Copyright { threshold_after => 5 };
+
+=head2 threshold, threshold_before, threshold_after
+
+Stop parsing after this many lines whithout copyright information,
+before or after having found any copyright information at all.
+C<threshold> sets both C<threshold_before> and C<threshold_after>.
+
+By default unset: All lines are parsed.
+
+=head1 FUNCTIONS
+
+Exports one function: C<copyright>.
+This module uses L<Exporter::Tiny> to export functions,
+which allows for flexible import options;
+see the L<Exporter::Tiny> documentation for details.
+
 =cut
 
-my $blank_re = qr/[ \t]/;
-my $sign_re  = qr/copyright(?:-holders?)?|copr\.|[©⒞Ⓒⓒ🄒🄫🅒]/i;
+my $blank_re          = qr/[ \t]/;
+my $blank_or_break_re = qr/$blank_re+|$blank_re*\n$blank_re*/;
+my $sign_re = qr/copyright(?:-holders?)?|copr\.|[©⒞Ⓒⓒ🄒🄫🅒]/i;
 my $pseudo_sign_re = qr/\(c\)/i;
 
-my $year_re  = qr/\b[0-9]{4}\b/;
-my $comma_re = qr/[,\s]*/;
-my $dash_re  = qr/$blank_re*[-˗‐‑‒–—―⁃−﹣－]$blank_re*/;
-my $owner_initial_re = qr/[^\s!\"#$%&'()*+.\/:;<=>?@[\\\]^_`{|}~]/;
+my $year_re = qr/\b[0-9]{4}\b/;
+my $comma_re
+	= qr/$blank_re*,$blank_or_break_re|$blank_or_break_re,$blank_re*|$blank_re*,$blank_re*|$blank_or_break_re|$blank_or_break_re/;
+my $dash_re
+	= qr/$blank_re*[-˗‐‑‒–—―⁃−﹣－]$blank_or_break_re*/;
+my $owner_initial_re = qr/[^\s!\"#$%&'()*+,.\/:;<=>?@[\\\]^_`{|}~]/;
 
 # TODO: test if \K or non-backref beneficial on perl >= 5.10
 #my $comma_tidy_re = qr/$year_re\K$comma_re(?=$year_re)/;
 #my $comma_tidy_re = qr/($year_re)$comma_re(?=$year_re)/;
 my $comma_tidy_re = qr/(?<=$year_re)$comma_re(?=$year_re)/;
+my $comma_tidied  = ', ';
 
 # TODO: test if \K beneficial on perl >= 5.10
 #my $year_begin_re = qr/(?:\A|(?<!-))($year_re)\K/;
@@ -98,42 +123,94 @@ my $y_save_re   = qr/(?:(?<=[^-]($year_re))|(?<=\A($year_re)))(?{$y=$^N})/;
 my $y_next_re   = qr/(??{++$y})/;
 my $y_future_re = qr/(??{if($y<=$^N){$y=$^N;'';}else{'XXXX';}})/;
 my $years_tidy_re
-	= qr/$y_save_re(?:(?:$dash_re|$comma_re)($y_next_re)\b|-($year_re)$y_future_re\b)+/;
+	= qr/$y_save_re(?:(?:$dash_re|$comma_tidied)($y_next_re)\b|-($year_re)$y_future_re\b)+/;
 
-# Stop-chars tested searching https://codesearch.debian.net/ for these:
-# (?i)copyright (?:(?:claim|holder|info|information|notice|owner|ownership|statement|string)s?|in|is|to)[^a-zA-Z0-9!\"$%&\'()*,.\/:;<=>?\[\\\]|} \t].
+# this should cause *no* false positives, and stop-chars therefore
+# exclude e.g. email address building blocks; tested against the code
+# corpus at https://codesearch.debian.net/ (tricky: its RE2 engine lacks
+# support for negative groups) using searches like these:
 # (?i)copyright (?:(?:claim|holder|info|information|notice|owner|ownership|statement|string)s?|in|is|to)@\w
+# (?i)copyright (?:(?:claim|holder|info|information|notice|owner|ownership|statement|string)s?|in|is|to)@\b[-_@]
+# (?im)copyright (?:(?:claim|holder|info|information|notice|owner|ownership|statement|string)s?|in|is|to)[^ $]
 my $chatter_re
-	= qr/copyright (?:(?:claim|holder|info|information|notice|owner|ownership|statement|string)s?|in|is|to)(?:\b[^@]|@\W)/i;
+	= qr/copyright (?:(?:claim|holder|info|information|law|notice|owner|ownership|statement|string)s?|and|in|is|on|to)(?:$|@\W|[^a-zA-Z0-9@_-])/im;
 
 my $signs_re
-	= qr/(?:$sign_re|(?:^|\s)$pseudo_sign_re)(?:$blank_re+(?:$sign_re|$pseudo_sign_re))*/;
+	= qr/(?:$sign_re|(?:^|$blank_re)$pseudo_sign_re)(?:$blank_re+(?:$sign_re|$pseudo_sign_re))*/m;
 my $years_re = qr/((?:$year_re(?:(?:$dash_re|$comma_re)$year_re)*)?)/;
 my $owners_re
-	= qr/((?:\bby\b\s*)?((?:$owner_initial_re\S*(?:$blank_re*\S+)*)?))/;
+	= qr/((?:\bby$blank_or_break_re)?((?:$owner_initial_re\S*(?:$blank_re*\S+)*)?))/;
 
-my $years_owners_re = qr/$comma_re$years_re$comma_re$owners_re/;
 my $sign_years_owners_re
-	= qr/(?:$chatter_re.*|$signs_re:?$comma_re$years_owners_re)/;
+	= qr/(?:$chatter_re.*|$signs_re(?::$blank_or_break_re|$comma_re)$years_re$comma_re?$owners_re|(?:\n|\z))/s;
 
-sub copyright
+sub _generate_copyright
 {
-	my $copyright = shift;
-	Carp::croak("String::Copyright strings require defined parts")
-		unless 1 + @_ == grep {defined} $copyright, @_;
+	my ( $class, $name, $args, $globals ) = @_;
 
-	# String::Copyright objects are effectively immutable and can be reused
-	if ( !@_ && ref($copyright) eq __PACKAGE__ ) {
-		return $copyright;
-	}
+	return sub {
+		my $copyright = shift;
 
-	# stringify objects
-	$copyright = "$copyright";
+		Carp::croak("String::Copyright strings require defined parts")
+			unless 1 + @_ == grep {defined} $copyright, @_;
 
-	# TODO: also parse @_ - but each separately!
-	my $blocks = parse_string( split /^/, $copyright );
+	   # String::Copyright objects are effectively immutable and can be reused
+		if ( !@_ && ref($copyright) eq __PACKAGE__ ) {
+			return $copyright;
+		}
 
-	bless [ $copyright, $blocks ], __PACKAGE__;
+		# stringify objects
+		$copyright = "$copyright";
+
+		# TODO: also parse @_ - but each separately!
+		my @block;
+		my $skipped = 0;
+		while ( $copyright =~ /$sign_years_owners_re/g ) {
+			my $years        = $1;
+			my $owners_dirty = $2;
+			my $owners       = $3 || '';
+
+			if ( $globals->{threshold_before} || $globals->{threshold} ) {
+				last
+					if (!@block
+					and !length $years
+					and !length $owners
+					and ++$skipped >= ( $globals->{threshold_before}
+							|| $globals->{threshold} ) );
+			}
+			if ( $globals->{threshold_after} || $globals->{threshold} ) {
+
+				# "after" detects end of _current_ line so is skewed by one
+				last
+					if (@block
+					and !length $years
+					and !length $owners
+					and ++$skipped >= 1
+					+ ( $globals->{threshold_after} || $globals->{threshold} )
+					);
+			}
+			next if ( !length $years and !length $owners );
+			$skipped = 0;
+
+			# normalize
+			$years =~ s/$comma_tidy_re/$comma_tidied/g;
+			$years =~ s/$years_tidy_re/-$^N/g;
+			$owners =~ s/\s{2,}/ /g;
+
+			my $copyright_match
+				= ( length $years and length $owners )
+				? $years . ' ' . $owners
+				: $years . $owners;
+
+# split owner into owner_id and owner
+
+			push @block, $copyright_match;
+		}
+
+# TODO: save $skipped_lines to indicate how dirty parsing was
+
+		bless [ $copyright, \@block ], __PACKAGE__;
+		}
 }
 
 sub new
@@ -172,50 +249,6 @@ sub _compose
 
 sub is_normalized { !defined $_[0]->[PLAINTEXT] }
 
-sub parse_string
-{
-	my @block;
-	my $skipped_lines = 0;
-
-	while (@_) {
-		my ( $years, $owners_dirty, $owners )
-			= shift =~ /$sign_years_owners_re/;
-
-		unless ( length $years or length $owners ) {
-
- # skip remaining lines if a copyright blocks was found more than 5 lines ago.
- # so a copyright block may contain up to 5 blank lines, but no more
-			last if ( @block and $skipped_lines++ > 5 );
-			next;
-		}
-
-		# extend block to next line(s) if only years found so far
-		while ( @_ and length $years and not length $owners_dirty ) {
-			( my $newyears, $owners_dirty, $owners )
-				= shift =~ /^$years_owners_re/;
-			$years .= ' ' . $newyears if length $newyears;
-		}
-
-		# normalize
-		$years =~ s/$comma_tidy_re/, /g;
-		$years =~ s/$years_tidy_re/-$^N/g;
-		$owners =~ s/\s{2,}/ /g;
-
-		my $copyright_match
-			= ( length $years and length $owners )
-			? $years . ' ' . $owners
-			: $years . $owners;
-
-# split owner into owner_id and owner
-
-		push @block, $copyright_match;
-	}
-
-# TODO: save $skipped_lines to indicate how dirty parsing was
-
-	return \@block;
-}
-
 =head1 SEE ALSO
 
 =over 4
@@ -224,6 +257,10 @@ sub parse_string
 
 L<Encode>
 
+=item *
+
+L<Exporter::Tiny>
+
 =back
 
 =head1 BUGS/CAVEATS/etc
